from django.shortcuts import render_to_response
from django.http import HttpResponse, HttpResponseRedirect
from django.contrib.auth.decorators import login_required
from django.template import RequestContext

from project.main.models import Controller


#--------------------------------------------------------------------------------------------------

@login_required
def list(request):

    if not request.user.permissions.can_manage_controllers:
        return HttpResponseRedirect("/")

    items = Controller.objects.all().order_by("nad")

    return render_to_response("controller/list.html",
        {
            "items" : items,
            "active_menu": "controllers",
        }, context_instance = RequestContext(request)
    )

#--------------------------------------------------------------------------------------------------

@login_required
def edit(request, id = 0):

    if not request.user.permissions.can_manage_controllers:
        return HttpResponseRedirect("/")

    id = int(id)
    new_item = id == 0

    if not new_item:
        try:
            item = Controller.objects.get(id = id)
        except:
            new_item = True

    if new_item:
        item = Controller()
        item.created = request.user

    if request.method == "POST":

        post = lambda key : request.POST.get(key, "")

        if post("operation") == "delete":
            item.delete()
        else:
            item.nad = post("nad")
            item.location = post("location")
            item.function = post("function")
            item.owner_id = post("owner")
            item.save()

        return HttpResponseRedirect("/controller/list/")

    return render_to_response("controller/edit.html",
        {
            "item": item,
            "active_menu": "controllers",
        }, context_instance = RequestContext(request)
    )

#--------------------------------------------------------------------------------------------------

@login_required
def delete(request, id):

    if not request.user.permissions.can_manage_controllers:
        return HttpResponseRedirect("/")

    try:
        Controller.objects.get(id = id).delete()
    except:
        pass

    return HttpResponseRedirect("/controller/list/")

#--------------------------------------------------------------------------------------------------

@login_required
def toggle_active(request, id):

    if not request.user.permissions.can_manage_controllers:
        return HttpResponseRedirect("/")

    try:
        item = Controller.objects.get(id = id)
        item.active = not item.active
        item.save()
    except:
        pass

    return HttpResponseRedirect("/controller/list/")

#--------------------------------------------------------------------------------------------------

@login_required
def get_available_controller_list(request):

    if request.user.permissions.is_server_admin:
        items = Controller.objects.filter(plant = None)
    else:
        items = Controller.objects.filter(owner = request.user, plant = None)

    return render_to_response("controller/available_controller_list.html",
        {
            "items": items.order_by("nad"),
        }, context_instance = RequestContext(request)
    )

#--------------------------------------------------------------------------------------------------

@login_required
def get_controller_list_row(request, id):

    try:
        return render_to_response("controller/controller_list_row.html",
            {
                "c": Controller.objects.get(id = id),
            }, context_instance = RequestContext(request)
        )
    except:
        return HttpResponse("")

#--------------------------------------------------------------------------------------------------

def nad_valid(request, nad):
    try:
        valid = Controller.objects.filter(nad=nad).count() == 0
    except:
        valid = False
    return HttpResponse("1" if valid else "0")

#--------------------------------------------------------------------------------------------------