from django.shortcuts import render_to_response
from django.http import HttpResponse, HttpResponseRedirect
from django.contrib import auth
from django.contrib.auth.decorators import login_required
from django.contrib.auth.models import User
from django.template import RequestContext

import project.settings
from project.main.models import Page, UpdatePeriod, Plant


#--------------------------------------------------------------------------------------------------

def render_page(page, rw_access):

    import project.settings
    from django.core.cache import cache

    # if cache is enabled
    if project.settings.PAGE_CACHE_VALIDITY != 0:
        cache_key = page.get_cache_key(rw_access)
        content = cache.get(cache_key)

        if content != None:
            return content

    from project.main.models import Template
    import project.main.object_factory
    import re

    update_period = 10000

    if page.update_period != None:
        update_period = page.update_period.get_period_ms()

    #----------------------------------------------------------------------------------------------

    def parse_template_tag(match):

        params = match.group(1)

        p = re.compile(r'<name>(.*?)</name>', re.DOTALL | re.IGNORECASE)
        m = p.search(params)
        if m != None:
            replaces = []
            try:
                template = Template.objects.get(name = m.group(1), published = True)

                p = re.compile(r'<search>(.*?)</search>.*?<replace>(.*?)</replace>', re.DOTALL | re.IGNORECASE)
                matches = p.findall(params)

                if matches:
                    for m in matches:
                        replaces.append((m[0], m[1]))
            except:
                return ""

            content = template.content
            for r in replaces:
                content = content.replace(r[0], r[1])

            return content
        else:
            return ""

    #----------------------------------------------------------------------------------------------

    def parse_cybro_tag(match):

        params = match.group(1)

        p = re.compile(r'<type>(.*?)</type>', re.DOTALL | re.IGNORECASE)
        m = p.search(params)

        if m != None:
            object_type = m.group(1)
            obj = factory.create_object(object_type, params, rw_access)
            if obj != None:
                return obj.create_object_html()

        return ""

    #----------------------------------------------------------------------------------------------

    factory = project.main.object_factory.ObjectFactory(page.get_plant(), rw_access)

    p = re.compile(r'<template>(.*?)</template>', re.DOTALL | re.IGNORECASE)
    template_html = p.sub(parse_template_tag, page.content)

    p = re.compile(r'<cybro>(.*?)</cybro>', re.DOTALL | re.IGNORECASE)
    html = p.sub(parse_cybro_tag, template_html)

    content = html + factory.create_html(update_period)

    if project.settings.PAGE_CACHE_VALIDITY != 0:
        cache.set(cache_key, content, project.settings.PAGE_CACHE_VALIDITY)

    return content

#--------------------------------------------------------------------------------------------------

def show(request, id):

    try:
        id = int(id)
    except:
        id = 1

    try:
        page = Page.objects.get(id = id)
    except:
        if id == 1:
            raise
        else:
            return HttpResponseRedirect("/")

    if not page.can_user_access(request.user):
        return HttpResponseRedirect("/")

    children = []
    if len(page.content) != 0:
        rw_access = request.user.is_authenticated() and request.user.has_rw_access(page)
        page_content = render_page(page, rw_access)
    else:
        page_content = ""
        for p in page.get_children():
            if p.can_user_access(request.user):
                children.append(p)

    if page.can_user_edit(request.user):
        plant = page.get_plant()
        plant_id = plant.id if plant != None else 0
        page_edit_link = "/page/edit/%d/%d/" % (plant_id, page.id)
    else:
        page_edit_link = ""

    if id == 1:
        if request.user.is_authenticated() and request.user.permissions.can_manage_plants:
            plants = Plant.objects.all().order_by("order")
        else:
            from django.db.models import Q
            query = Q(public = True)
            if request.user.is_authenticated():
                query |=  Q(admins__in = [request.user]) | Q(users__in = [request.user])
            plants = Plant.objects.filter(query).distinct().order_by("order")
    else:
        plants = None

    return render_to_response("page/show.html",
        {
            "page": page,
            "page_content": page_content,
            "current_page": page,
            "page_edit_link": page_edit_link,
            "children": children,
            "plants": plants,
        }, context_instance = RequestContext(request)
    )

#--------------------------------------------------------------------------------------------------

@login_required
def list(request):

    return HttpResponse("")
    #if request.user.is_admin():
        #items = request.user.get_all_site_pages()
    #else:
        #if request.user.permissions.can_manage_content:
            #items = request.user.get_all_pages()
        #else:
            #return HttpResponseRedirect("/")

    #return render_to_response("page/list.html",
        #{
            #"editing_site_content": request.user.is_admin(),
            #"items" : items,
        #}, context_instance = RequestContext(request)
    #)

#--------------------------------------------------------------------------------------------------

@login_required
def plant_page_list(request, id):
    try:
        plant = Plant.objects.get(id = id)

        if not request.user.can_manage_plant(plant):
            raise
    except:
        return HttpResponseRedirect("/")

    items = [plant.homepage] + plant.homepage.get_descendants()

    return render_to_response("page/list.html",
        {
            "editing_site_content": False,
            "items" : items,
            "plant": plant,
            "plant_id": plant.id,
            "active_menu": "plants",
        }, context_instance = RequestContext(request)
    )

#--------------------------------------------------------------------------------------------------

@login_required
def site_list(request):

    if not request.user.permissions.can_manage_site_content:
        return HttpResponseRedirect("/")

    return render_to_response("page/list.html",
        {
            "editing_site_content": True,
            "items" : request.user.get_all_site_pages(),
            "plant_id": 0,
            "active_menu": "site_content",
        }, context_instance = RequestContext(request)
    )

#--------------------------------------------------------------------------------------------------

@login_required
def edit(request, plant_id, id = 0):

    plant_id = int(plant_id)
    id = int(id)
    plant = None
    new_item = id == 0
    site_page = plant_id == 0

    if not new_item:
        try:
            item = Page.objects.get(id = id)
        except:
            new_item = True

    try:
        plant = Plant.objects.get(id = plant_id)

        if not plant.can_user_manage_content(request.user):
            return HttpResponseRedirect("/")
    except Exception, e:
        # site content
        if not request.user.permissions.can_manage_site_content:
            return HttpResponseRedirect("/")

    if new_item:
        item = Page()
        if plant != None:
            item.plant = plant
            item.parent = item.plant.homepage
        else:
            item.plant_id = 0

    editing_root_page = not (new_item or item.parent_id != 0)

    if request.method == "POST":
        operation = request.POST.get("operation", "")

        if operation == "delete":
            item.delete()
        else:
            try:
                period = UpdatePeriod.objects.get(id = request.POST.get("update_period", "0"))
            except:
                period = None

            item.name = request.POST.get("name", "")
            item.description = request.POST.get("description", "")
            item.content = request.POST.get("content", "")
            item.update_period = period
            item.modified_by = request.user

            if not editing_root_page:
                item.parent_id = request.POST.get("parent", "0")

            if new_item:
                item.author = request.user
                item.order = Page.objects.count()

            item.save()
            item.delete_cache()

            if request.POST.get("order_changed", "") == "1":
                n = 0
                while True:
                    id = int(request.POST.get("order_%d" %n, "-1"))

                    if id == 0:
                        id = item.id

                    if id != -1:
                        Page.objects.filter(id = id).update(order = n)
                    else:
                        break
                    n += 1

        backlink = "/page/site_list/" if site_page else "/plant/page_list/%d/" % plant_id

        if operation == "preview":
            return HttpResponseRedirect(item.get_link())
        elif operation == "back_to_manage":
            return HttpResponseRedirect(backlink)
        else:
            pass

    from project.main.object_factory import ObjectFactory

    if new_item:
        item.author = request.user
        if site_page:
            item.update_period_id = 7 # never
        else:
            try:
                item.update_period = UpdatePeriod.objects.get(period = 60)
            except:
                pass

    # delete current path for media browser
    if request.session.has_key("current_path"):
        del request.session["current_path"]

    return render_to_response("page/edit.html",
        {
            "item": item,
            "is_site_page": site_page,
            "object_factory": ObjectFactory(item.get_plant()),
            "enable_order_edit": item.get_siblings_with_self().count() > 1,
            "update_periods": UpdatePeriod.objects.all().order_by("period"),
            "editing_root_page": editing_root_page,
            "plant_id": plant_id,
            "active_menu": "site_content" if site_page else "plants",
            "enable_tinymce": False,
        }, context_instance = RequestContext(request)
    )

#--------------------------------------------------------------------------------------------------

@login_required
def get_order_area(request, parent_page, current_page):

    parent_page = int(parent_page)
    current_page = int(current_page)
    current_item = None
    new_item = current_page == 0

    if not new_item:
        try:
            current_item = Page.objects.get(id = current_page)
        except:
            new_item = True

    if new_item:
        current_item = Page(id = 0, name = "[ new page ]")

    items = Page.objects.filter(parent__id = parent_page).order_by("order")

    if len(items) != 0:
        return render_to_response("page/edit_order.html",
            {
                "items": items,
                "current_item": current_item,
                "add_item": current_item.id == 0 or current_item.parent.id != parent_page,
            }, context_instance = RequestContext(request)
        )
    else:
        return HttpResponse("")

#--------------------------------------------------------------------------------------------------

@login_required
def delete(request, plant_id = 0, id = 0):

    plant_id = int(plant_id)

    try:
        plant = Plant.objects.get(id = plant_id)

        if not plant.can_user_manage_content(request.user):
            return HttpResponseRedirect("/")
    except Exception, e:
        # site content
        if not request.user.permissions.can_manage_site_content:
            return HttpResponseRedirect("/")

    try:
        item = Page.objects.get(id = id)

        for p in item.get_descendants():
            p.delete()

        item.delete()
    except:
        pass

    return HttpResponseRedirect("/page/site_list/" if plant_id == 0 else "/plant/page_list/%d/" % plant_id)

#--------------------------------------------------------------------------------------------------

@login_required
def toggle_public(request, plant_id = 0, id = 0):

    plant_id = int(plant_id)

    try:
        plant = Plant.objects.get(id = plant_id)

        if not plant.can_user_manage_content(request.user):
            return HttpResponseRedirect("/")
    except Exception, e:
        # site content
        if not request.user.permissions.can_manage_site_content:
            return HttpResponseRedirect("/")

    try:
        item = Page.objects.get(id = id)
        item.public = not item.public
        item.save()
    except:
        pass

    return HttpResponseRedirect("/page/site_list/" if plant_id == 0 else "/plant/page_list/%d/" % plant_id)

#--------------------------------------------------------------------------------------------------

@login_required
def toggle_published(request, plant_id = 0, id = 0):

    plant_id = int(plant_id)

    try:
        plant = Plant.objects.get(id = plant_id)

        if not plant.can_user_manage_content(request.user):
            return HttpResponseRedirect("/")
    except Exception, e:
        # site content
        if not request.user.permissions.can_manage_site_content:
            return HttpResponseRedirect("/")

    try:
        item = Page.objects.get(id = id)
        item.published = not item.published
        item.save()
    except:
        pass

    return HttpResponseRedirect("/page/site_list/" if plant_id == 0 else "/plant/page_list/%d/" % plant_id)

#--------------------------------------------------------------------------------------------------
