import os
import deploy
import settings_local

PROJECT_PATH = os.path.realpath(os.path.dirname(__file__))

DEBUG = not deploy.WEB_DEPLOY
TEMPLATE_DEBUG = DEBUG
DEBUG_TOOLBAR = False

if not DEBUG:
    DEBUG_TOOLBAR = False


ADMINS = settings_local.ADMINS

MANAGERS = ADMINS

DATABASE_ENGINE = settings_local.DATABASE_ENGINE
DATABASE_NAME = settings_local.DATABASE_NAME
DATABASE_USER = settings_local.DATABASE_USER
DATABASE_PASSWORD = settings_local.DATABASE_PASSWORD
DATABASE_HOST = settings_local.DATABASE_HOST
DATABASE_PORT = settings_local.DATABASE_PORT

EMAIL_HOST = settings_local.SMTP_HOST
EMAIL_HOST_USER = settings_local.SMTP_HOST_USER
EMAIL_HOST_PASSWORD = settings_local.SMTP_HOST_PASSWORD
EMAIL_PORT = settings_local.SMTP_PORT
EMAIL_USE_TLS = settings_local.SMTP_USE_TLS
DEFAULT_FROM_EMAIL = settings_local.DEFAULT_FROM_EMAIL
SERVER_EMAIL = settings_local.SERVER_EMAIL
REPORT_FROM_EMAIL = settings_local.REPORT_FROM_EMAIL
REPORT_EMAIL_SUBJECT = settings_local.REPORT_EMAIL_SUBJECT

try:
    CSV_EXPORT_CHARSET = settings_local.CSV_EXPORT_CHARSET
except NameError:
     CSV_EXPORT_CHARSET = None
except AttributeError:
     CSV_EXPORT_CHARSET = None


# Local time zone for this installation. Choices can be found here:
# http://en.wikipedia.org/wiki/List_of_tz_zones_by_name
# although not all choices may be available on all operating systems.
# If running in a Windows environment this must be set to the same as your
# system time zone.
TIME_ZONE = settings_local.TIME_ZONE

# Language code for this installation. All choices can be found here:
# http://www.i18nguy.com/unicode/language-identifiers.html
LANGUAGE_CODE = "en"

LANGUAGES = (
    ("en", "English"),
)

SITE_ID = 1

# If you set this to False, Django will make some optimizations so as not
# to load the internationalization machinery.
USE_I18N = False

# URL that handles the media served from MEDIA_ROOT. Make sure to use a
# trailing slash if there is a path component (optional in other cases).
# Examples: "http://media.lawrence.com", "http://example.com/media/"
MEDIA_URL = ''

# URL prefix for admin media -- CSS, JavaScript and images. Make sure to use a
# trailing slash.
# Examples: "http://foo.com/media/", "/media/".
#ADMIN_MEDIA_PREFIX = '/media/'

# Make this unique, and don't share it with anybody.
SECRET_KEY = settings_local.SECRET_KEY

# List of callables that know how to import templates from various sources.
TEMPLATE_LOADERS = (
    'django.template.loaders.filesystem.load_template_source',
    'django.template.loaders.app_directories.load_template_source',
#     'django.template.loaders.eggs.load_template_source',
)


MIDDLEWARE_CLASSES = (
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    #'pagination.middleware.PaginationMiddleware',
)

if DEBUG_TOOLBAR:
    MIDDLEWARE_CLASSES += (
    'debug_toolbar.middleware.DebugToolbarMiddleware',
)


ROOT_URLCONF = 'project.urls'

TEMPLATE_DIRS = (
    # Put strings here, like "/home/html/django_templates" or "C:/www/django/templates".
    # Always use forward slashes, even on Windows.
    # Don't forget to use absolute paths, not relative paths.
    os.path.join(PROJECT_PATH, 'template'),
)

TEMPLATE_CONTEXT_PROCESSORS = (
    "django.core.context_processors.auth",
    "django.core.context_processors.debug",
    #"django.core.context_processors.i18n",
    "django.core.context_processors.media",
    "django.core.context_processors.request",
    "project.main.context_processors.get_site_name",
    "project.main.context_processors.get_contact_email",
    "project.main.context_processors.get_head_caption",
    "project.main.context_processors.get_footer_text",
    "project.main.context_processors.get_google_webmaster_tools_code",
    "project.main.context_processors.get_shortcut_icon",
)

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.admin',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
)

if DEBUG_TOOLBAR:
    INSTALLED_APPS += (
        'debug_toolbar',
    )

INSTALLED_APPS += (
    'project.main',
    'project.accounts',
    #'pagination',
)

CUSTOM_USER_MODEL = 'main.CustomUser'

AUTHENTICATION_BACKENDS = (
    'project.accounts.backends.CustomUserModelBackend',
    'django.contrib.auth.backends.ModelBackend',
)

#*************************************************************
# Django Debug Toolbar

if DEBUG_TOOLBAR:
    INTERNAL_IPS = ('127.0.0.1',)

    DEBUG_TOOLBAR_PANELS = (
        #'debug_toolbar.panels.version.VersionDebugPanel',
        'debug_toolbar.panels.timer.TimerDebugPanel',
        #'debug_toolbar.panels.settings_vars.SettingsVarsDebugPanel',
        #'debug_toolbar.panels.headers.HeaderDebugPanel',
        #'debug_toolbar.panels.request_vars.RequestVarsDebugPanel',
        #'debug_toolbar.panels.template.TemplateDebugPanel',
        'debug_toolbar.panels.sql.SQLDebugPanel',
        #'debug_toolbar.panels.signals.SignalDebugPanel',
        #'debug_toolbar.panels.logger.LoggingPanel',
    )

    DEBUG_TOOLBAR_CONFIG = ({
        'INTERCEPT_REDIRECTS' : False,
    })


#*************************************************************
# Application settings


MEDIA_URL = "/static"
DATA_URL = "/data"
CACHE_URL = "/data"

SCGI_URL = settings_local.SCGI_URL
SCGI_PORT = settings_local.SCGI_PORT
SCGI_HOST = "localhost"
PAGE_CACHE_VALIDITY = settings_local.PAGE_CACHE_VALIDITY
DATA_FOLDER_ROOT = settings_local.DATA_DIRECTORY + "media/"
MMANAGER_THUMBS_ROOT = settings_local.DATA_DIRECTORY + "thumbs/"
CACHE_ROOT = settings_local.DATA_DIRECTORY + "cache/"
CACHE_BACKEND = "file://" + settings_local.DATA_DIRECTORY + "cache/"
DATA_ROOT = settings_local.DATA_DIRECTORY
MEDIA_ROOT = settings_local.DATA_DIRECTORY
CYBRO_SCGI_SERVER_CONFIG = settings_local.CYBRO_SCGI_SERVER_DIRECTORY + "scgi_server.ini"
CYBRO_SCGI_SERVER_LOG_PATH = settings_local.CYBRO_SCGI_SERVER_DIRECTORY + "log/"
DATALOGGER_CONFIG = settings_local.CYBRO_SCGI_SERVER_DIRECTORY + "data_logger.xml"
SITE_URL = settings_local.SITE_URL
ADMIN_MEDIA_PREFIX = settings_local.ADMIN_MEDIA_PREFIX
STATIC_SETTINGS = settings_local.STATIC_SETTINGS


if not deploy.WEB_DEPLOY:
    CACHES = {
        'default': {
            'BACKEND': 'django.core.cache.backends.dummy.DummyCache',
        }
    }